﻿using System;
using System.IO;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.Storage.Streams;
using System.Runtime.InteropServices.WindowsRuntime;
using ImageEffectsWP8;
using System.Windows;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Controls;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.ApplicationModel.Core;
using Windows.UI.Core;

namespace SimpleImageViewWP8
{
    internal class PictureService
    {
        /// <summary>
        /// The effects engine
        /// </summary>
        Lazy<ImageEffects> Effects;

        /// <summary>
        /// Initializes a new instance of the <see cref="PictureService"/> class.
        /// </summary>
        public PictureService()
        {
            Effects = new Lazy<ImageEffects>();
        }

        /// <summary>
        /// Apply a gray scale effect to the image.
        /// </summary>
        /// <param name="image">The image.</param>
        public async void ToGrayScale(Image image)
        {
            StorageFile file = await StorageFile.GetFileFromApplicationUriAsync(new Uri(string.Format("ms-appx:///Assets/{0}", App.ImageFileName)));

            using (IRandomAccessStream fileStream = await file.OpenAsync(Windows.Storage.FileAccessMode.Read))
            {
                BitmapDecoder decoder = await BitmapDecoder.CreateAsync(fileStream);

                PixelDataProvider pixelData = await decoder.GetPixelDataAsync(BitmapPixelFormat.Bgra8,    
                                                                              BitmapAlphaMode.Straight,
                                                                              new BitmapTransform(),
                                                                              ExifOrientationMode.IgnoreExifOrientation, 
                                                                              ColorManagementMode.DoNotColorManage);

                var pixels = pixelData.DetachPixelData();

                WriteableBitmap wb = new WriteableBitmap((int)decoder.PixelWidth, (int)decoder.PixelHeight);

                Effects.Value.ApplygGrayscale(pixels.AsBuffer()).Completed = (info, status) =>
                                    {
                                        using (Stream stream = wb.PixelBuffer.AsStream())
                                        {
                                            stream.WriteAsync(pixels, 0, pixels.Length).Wait();
                                        }

                                        var dummy = CoreApplication.MainView.CoreWindow.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => 
                                        {
                                            UpdateImageSource(image, wb);
                                        });
                                    };
                
            }
        }

        /// <summary>
        /// Applies the brightness scale to the image.
        /// </summary>
        /// <param name="image">The image.</param>
        /// <param name="brightness">The brightness scale (0->255).</param>
        public async void ApplyBrightnessScale(Image image, int brightness)
        {
            StorageFile file = await StorageFile.GetFileFromApplicationUriAsync(new Uri(string.Format("ms-appx:///Assets/{0}", App.ImageFileName)));

            using (IRandomAccessStream fileStream = await file.OpenAsync(Windows.Storage.FileAccessMode.Read))
            {
                BitmapDecoder decoder = await BitmapDecoder.CreateAsync(fileStream);

                PixelDataProvider pixelData = await decoder.GetPixelDataAsync(BitmapPixelFormat.Bgra8,
                                                                              BitmapAlphaMode.Straight,
                                                                              new BitmapTransform(),
                                                                              ExifOrientationMode.IgnoreExifOrientation,
                                                                              ColorManagementMode.DoNotColorManage);

                var pixels = pixelData.DetachPixelData();

                WriteableBitmap wb = new WriteableBitmap((int)decoder.PixelWidth, (int)decoder.PixelHeight);

                Effects.Value.ApplyBrightnessScale(pixels.AsBuffer(), brightness).Completed = (info, status) =>
                                    {
                                        using (Stream stream = wb.PixelBuffer.AsStream())
                                        {
                                            stream.WriteAsync(pixels, 0, pixels.Length).Wait();
                                        }

                                        var dummy = CoreApplication.MainView.CoreWindow.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                                        {
                                            UpdateImageSource(image, wb);
                                        }).AsTask();
                                    };

            }
        }

        /// <summary>
        /// Gets the PNG engine version.
        /// </summary>
        /// <returns>The PNG engine version</returns>
        public string GetPngEngineVersion()
        {
            return Effects.Value.GetEngineVersion("png");
        }

        /// <summary>
        /// Updates the image source.
        /// </summary>
        /// <param name="image">The image to update.</param>
        /// <param name="source">The source for the image.</param>
        private void UpdateImageSource(Image image, WriteableBitmap source)
        {
            image.Source = source;
        }
    }
}
